@@ -143,38 +143,6 @@ extern "C" {
     /* Standard Gtk function */
     GType eel_canvas_rect_get_type (void) G_GNUC_CONST;
 
-
-    /* Ellipse item.  No configurable or queryable arguments are available (use those in
-     * EelCanvasRE).
-     */
-
-
-#define EEL_TYPE_CANVAS_ELLIPSE            (eel_canvas_ellipse_get_type ())
-#define EEL_CANVAS_ELLIPSE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), EEL_TYPE_CANVAS_ELLIPSE, EelCanvasEllipse))
-#define EEL_CANVAS_ELLIPSE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), EEL_TYPE_CANVAS_ELLIPSE, EelCanvasEllipseClass))
-#define EEL_IS_CANVAS_ELLIPSE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EEL_TYPE_CANVAS_ELLIPSE))
-#define EEL_IS_CANVAS_ELLIPSE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EEL_TYPE_CANVAS_ELLIPSE))
-#define EEL_CANVAS_ELLIPSE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), EEL_TYPE_CANVAS_ELLIPSE, EelCanvasEllipseClass))
-
-
-    typedef struct _EelCanvasEllipse EelCanvasEllipse;
-    typedef struct _EelCanvasEllipseClass EelCanvasEllipseClass;
-
-    struct _EelCanvasEllipse
-    {
-        EelCanvasRE re;
-    };
-
-    struct _EelCanvasEllipseClass
-    {
-        EelCanvasREClass parent_class;
-    };
-
-
-    /* Standard Gtk function */
-    GType eel_canvas_ellipse_get_type (void) G_GNUC_CONST;
-
-
 #ifdef __cplusplus
 }
 #endif
