@@ -375,8 +375,8 @@ progress_widget_new (CajaProgressInfo *info)
     gtk_widget_set_size_request (label, 500, -1);
     gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
     gtk_label_set_line_wrap_mode (GTK_LABEL (label), PANGO_WRAP_WORD_CHAR);
-#if GTK_CHECK_VERSION (3, 14, 0)
-    gtk_widget_set_halign (label, GTK_ALIGN_START);
+#if GTK_CHECK_VERSION (3, 16, 0)
+    gtk_label_set_xalign (GTK_LABEL (label), 0.0);
 #else
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 #endif
@@ -401,7 +401,7 @@ progress_widget_new (CajaProgressInfo *info)
                        TRUE,TRUE,
                        0);
 
-    image = gtk_image_new_from_stock (GTK_STOCK_CANCEL,
+    image = gtk_image_new_from_icon_name ("gtk-cancel",
                                       GTK_ICON_SIZE_BUTTON);
     button = gtk_button_new ();
     gtk_container_add (GTK_CONTAINER (button), image);
@@ -417,8 +417,8 @@ progress_widget_new (CajaProgressInfo *info)
                         0);
 
     label = gtk_label_new ("details");
-#if GTK_CHECK_VERSION (3, 14, 0)
-    gtk_widget_set_halign (label, GTK_ALIGN_START);
+#if GTK_CHECK_VERSION (3, 16, 0)
+    gtk_label_set_xalign (GTK_LABEL (label), 0.0);
 #else
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 #endif
