@@ -560,6 +560,7 @@ caja_path_bar_get_preferred_width (GtkWidget *widget,
     gint child_height;
     gint height;
     gint child_min, child_nat;
+    gint slider_width;
 
     path_bar = CAJA_PATH_BAR (widget);
 
@@ -586,12 +587,20 @@ caja_path_bar_get_preferred_width (GtkWidget *widget,
     /* Theoretically, the slider could be bigger than the other button.  But we're
      * not going to worry about that now.
      */
-    path_bar->slider_width = MIN (height * 2 / 3 + 5, height);
+    gtk_widget_get_preferred_width (path_bar->down_slider_button,
+                                    &slider_width,
+                                    NULL);
+    gtk_widget_get_preferred_width (path_bar->up_slider_button,
+                                    &slider_width,
+                                    NULL);
+
 
     if (path_bar->button_list) {
-    	*minimum += (path_bar->spacing + path_bar->slider_width) * 2;
-    	*natural += (path_bar->spacing + path_bar->slider_width) * 2;
+        *minimum += (path_bar->spacing + slider_width) * 2;
+        *natural += (path_bar->spacing + slider_width) * 2;
     }
+    /*Let's keep the rest of this as it was */
+    path_bar->slider_width = slider_width;
 }
 
 static void
@@ -1597,26 +1606,29 @@ get_dir_name (ButtonData *button_data)
  */
 #if GTK_CHECK_VERSION(3,0,0)
 static void
-set_label_size_request (ButtonData *button_data)
+set_label_padding_size (ButtonData *button_data)
 {
     const gchar *dir_name = get_dir_name (button_data);
     PangoLayout *layout;
     gint width, height, bold_width, bold_height;
+    gint pad_left, pad_right;
     gchar *markup;
     
     layout = gtk_widget_create_pango_layout (button_data->label, dir_name);
     pango_layout_get_pixel_size (layout, &width, &height);
-  
+
     markup = g_markup_printf_escaped ("<b>%s</b>", dir_name);
     pango_layout_set_markup (layout, markup, -1);
     g_free (markup);
 
     pango_layout_get_pixel_size (layout, &bold_width, &bold_height);
 
-    gtk_widget_set_size_request (button_data->label,
-        			 MAX (width, bold_width),
-        			 MAX (height, bold_height));
-    
+    pad_left = (bold_width - width) / 2;
+    pad_right = (bold_width - width) / 2;
+
+    gtk_widget_set_margin_start (GTK_WIDGET (button_data->label), pad_left);
+    gtk_widget_set_margin_end (GTK_WIDGET (button_data->label), pad_right);
+
     g_object_unref (layout);
 }
 
@@ -1662,25 +1674,21 @@ caja_path_bar_update_button_appearance (ButtonData *button_data)
 
             markup = g_markup_printf_escaped ("<b>%s</b>", dir_name);
             gtk_label_set_markup (GTK_LABEL (button_data->label), markup);
-            g_free (markup);
+#if GTK_CHECK_VERSION(3,0,0)
+            gtk_widget_set_margin_end (GTK_WIDGET (button_data->label), 0);
+            gtk_widget_set_margin_start (GTK_WIDGET (button_data->label), 0);
+#endif
+            g_free(markup);
         }
         else
         {
             gtk_label_set_text (GTK_LABEL (button_data->label), dir_name);
+#if GTK_CHECK_VERSION(3,0,0)
+            set_label_padding_size (button_data);
+#endif
         }
     }
 
-#if GTK_CHECK_VERSION(3,0,0)
-    /* FIXME: Maybe we dont need this alignment at all and we can
-     * use GtkMisc aligments or even GtkWidget:halign/valign center.
-     *
-     * The following function ensures that the alignment will always
-     * request the same size whether the button's text is bold or not.
-     */
-    set_label_size_request (button_data);
-#endif
-
-
     if (button_data->image != NULL)
     {
         if (button_data->custom_icon)
