@@ -2238,6 +2238,9 @@ caja_property_browser_update_contents (CajaPropertyBrowser *property_browser)
     gboolean got_categories;
     char *name, *image, *type, *description, *display_name, *path, *mode;
     const char *text;
+#if GTK_CHECK_VERSION (3, 0, 0)
+    GtkStyleContext *context;
+#endif
 
     /* load the xml document corresponding to the path and selection */
     document = read_browser_xml (property_browser);
@@ -2255,6 +2258,8 @@ caja_property_browser_update_contents (CajaPropertyBrowser *property_browser)
     /* allocate a new container, with a scrollwindow and viewport */
     property_browser->details->content_frame = gtk_scrolled_window_new (NULL, NULL);
 #if GTK_CHECK_VERSION (3, 0, 0)
+    context = gtk_widget_get_style_context (property_browser->details->content_frame);
+    gtk_style_context_add_class (context, "frame");
     gtk_widget_set_vexpand (property_browser->details->content_frame, TRUE);
 #endif
     viewport = gtk_viewport_new (NULL, NULL);
